/*============================================================================
  File:     AnalyzingAndTroubleshootingQueryPlansAndJoins.sql

  Summary:  Query plan analysis and tuning joins.
  
  SQL Server Version: 2005+
------------------------------------------------------------------------------
  Written by Kimberly L. Tripp, SYSolutions, Inc.

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  This script is intended only as a supplement to demos and lectures
  given by Kimberly L. Tripp.  
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

-- These samples use the Credit database. You can download and restore the
-- credit database from here:
-- http://www.sqlskills.com/resources/conferences/CreditBackup80.zip

-- NOTE: This is a SQL Server 2000 backup and MANY examples will work on 
-- SQL Server 2000 in addition to SQL Server 2005.

USE [Credit];
GO

-- Let's start to analyze some queries and their performance:
SET STATISTICS IO ON;
GO

SELECT [c].[statement_no]
		, [s].[statement_dt]
		, [c].[charge_amt]
		, [p].[provider_name]
		, [m].[lastname]
	FROM [dbo].[charge] AS [c]
		INNER JOIN [dbo].[provider] AS [p]
			ON [p].[provider_no] = [c].[provider_no]
		INNER JOIN [dbo].[member] AS [m] 
			ON [c].[member_no] = [m].[member_no]
		INNER JOIN [dbo].[statement] AS [s] 
			ON [c].[statement_no] = [s].[statement_no]
		INNER JOIN [dbo].[region] AS [r] 
			ON [r].[region_no] = [m].[region_no]
WHERE [r].[region_name] = 'Japan'
	AND [c].[charge_amt] > 2500
OPTION (MAXDOP 1)
go

-------------------------------------------------------------------------------
-- First, let's get this plan as our base test case... review the 
-- showplan and FORCE every single index listed.
-------------------------------------------------------------------------------

-- USE:
--	0 = Table Scan
--	1 = Clustered Index Seek/Scan
--	name = for all non-clustered indexes 
--		(name is a bit safer)

SELECT [c].[statement_no]
		, [s].[statement_dt]
		, [c].[charge_amt]
		, [p].[provider_name]
		, [m].[lastname]
	FROM [dbo].[charge] AS [c] WITH (INDEX (1))
		INNER JOIN [dbo].[provider] AS [p] WITH (INDEX ([provider_ident]))
			ON [p].[provider_no] = [c].[provider_no]
		INNER JOIN [dbo].[member] AS [m] WITH (INDEX (1))
			ON [c].[member_no] = [m].[member_no]
		INNER JOIN [dbo].[statement] AS [s] WITH (INDEX (1))
			ON [c].[statement_no] = [s].[statement_no]
		INNER JOIN [dbo].[region] AS [r] WITH (INDEX (1))
			ON [r].[region_no] = [m].[region_no]
WHERE [r].[region_name] = 'Japan'
	AND [c].[charge_amt] > 2500
OPTION (MAXDOP 1)
GO

-------------------------------------------------------------------------------
-- Analyze the costs and find the problem table/join
-------------------------------------------------------------------------------

-- Where is all the cost? (Highest number in showplan output...)
	-- Should be in charge!

-- SO - what does the charge table need in this query?
-- SELECT List		statement_no, charge_amt
-- Join Cond1		*member_no
-- Join Cond2		provider_no
-- Join Cond3		statement_no
-- Search Arg		*charge_amt

-------------------------------------------------------------------------------
-- Review the green hint
-------------------------------------------------------------------------------

CREATE NONCLUSTERED INDEX MissingIndexDMVRecommendation
ON [dbo].[charge] ([charge_amt])
INCLUDE ([member_no],[provider_no],[statement_no]);
GO

-------------------------------------------------------------------------------
-- Review the DTA recommendations
-------------------------------------------------------------------------------

-- DTA will review A LOT more than just the "expensive" table, it will
-- review the entire query. Is that always good or always bad?! Let's see!

-- Highlight the query. Then, select Query, Analyze in DTA
CREATE NONCLUSTERED INDEX [dta_K6_K7_K3_K2] 
ON [dbo].[charge] ([member_no], [charge_amt], 
		[statement_no], [provider_no]);
GO

-- Let's do this a bit iteratively though... We're not going to blindly 
-- implement ALL of DTAs indexes. I'm going to start slowly - with 
-- just the indexes against our most expensive table:

-- Consider creating both and then see which one SQL Server picks?

-- BASELINE VERSION OF THE QUERY:
SELECT [c].[statement_no]
		, [s].[statement_dt]
		, [c].[charge_amt]
		, [p].[provider_name]
		, [m].[lastname]
	FROM [dbo].[charge] AS [c] WITH (INDEX (1))
		INNER JOIN [dbo].[provider] AS [p] WITH (INDEX (1))
			ON [p].[provider_no] = [c].[provider_no]
		INNER JOIN [dbo].[member] AS [m] WITH (INDEX (1))
			ON [c].[member_no] = [m].[member_no]
		INNER JOIN [dbo].[statement] AS [s] WITH (INDEX (1))
			ON [c].[statement_no] = [s].[statement_no]
		INNER JOIN [dbo].[region] AS [r] WITH (INDEX (1))
			ON [r].[region_no] = [m].[region_no]
WHERE [r].[region_name] = 'Japan'
	AND [c].[charge_amt] > 2500
OPTION (MAXDOP 1);
GO

-- SQL Server can do whatever it wants!
SELECT [c].[statement_no]
		, [s].[statement_dt]
		, [c].[charge_amt]
		, [p].[provider_name]
		, [m].[lastname]
	FROM [dbo].[charge] AS [c]
		INNER JOIN [dbo].[provider] AS [p]
			ON [p].[provider_no] = [c].[provider_no]
		INNER JOIN [dbo].[member] AS [m] 
			ON [c].[member_no] = [m].[member_no]
		INNER JOIN [dbo].[statement] AS [s] 
			ON [c].[statement_no] = [s].[statement_no]
		INNER JOIN [dbo].[region] AS [r] 
			ON [r].[region_no] = [m].[region_no]
WHERE [r].[region_name] = 'Japan'
	AND [c].[charge_amt] > 2500
OPTION (MAXDOP 1);
GO

-------------------------------------------------------------------------------
-- Test and compare the ALL of the options
-------------------------------------------------------------------------------

-- BASELINE VERSION OF THE QUERY:
SELECT [c].[statement_no]
		, [s].[statement_dt]
		, [c].[charge_amt]
		, [p].[provider_name]
		, [m].[lastname]
	FROM [dbo].[charge] AS [c] WITH (INDEX (1))
		INNER JOIN [dbo].[provider] AS [p] WITH (INDEX (1))
			ON [p].[provider_no] = [c].[provider_no]
		INNER JOIN [dbo].[member] AS [m] WITH (INDEX (1))
			ON [c].[member_no] = [m].[member_no]
		INNER JOIN [dbo].[statement] AS [s] WITH (INDEX (1))
			ON [c].[statement_no] = [s].[statement_no]
		INNER JOIN [dbo].[region] AS [r] WITH (INDEX (1))
			ON [r].[region_no] = [m].[region_no]
WHERE [r].[region_name] = 'Japan'
	AND [c].[charge_amt] > 2500
OPTION (MAXDOP 1);
GO

-- FORCE the "green hint" VERSION OF THE QUERY:
SELECT [c].[statement_no]
		, [s].[statement_dt]
		, [c].[charge_amt]
		, [p].[provider_name]
		, [m].[lastname]
	FROM [dbo].[charge] AS [c] WITH (INDEX (MissingIndexDMVRecommendation))
		INNER JOIN [dbo].[provider] AS [p] WITH (INDEX (1))
			ON [p].[provider_no] = [c].[provider_no]
		INNER JOIN [dbo].[member] AS [m] WITH (INDEX (1))
			ON [c].[member_no] = [m].[member_no]
		INNER JOIN [dbo].[statement] AS [s] WITH (INDEX (1))
			ON [c].[statement_no] = [s].[statement_no]
		INNER JOIN [dbo].[region] AS [r] WITH (INDEX (1))
			ON [r].[region_no] = [m].[region_no]
WHERE [r].[region_name] = 'Japan'
	AND [c].[charge_amt] > 2500
OPTION (MAXDOP 1);
GO

-- SQL Server can do whatever it wants!
SELECT [c].[statement_no]
		, [s].[statement_dt]
		, [c].[charge_amt]
		, [p].[provider_name]
		, [m].[lastname]
	FROM [dbo].[charge] AS [c]
		INNER JOIN [dbo].[provider] AS [p]
			ON [p].[provider_no] = [c].[provider_no]
		INNER JOIN [dbo].[member] AS [m] 
			ON [c].[member_no] = [m].[member_no]
		INNER JOIN [dbo].[statement] AS [s] 
			ON [c].[statement_no] = [s].[statement_no]
		INNER JOIN [dbo].[region] AS [r] 
			ON [r].[region_no] = [m].[region_no]
WHERE [r].[region_name] = 'Japan'
	AND [c].[charge_amt] > 2500
OPTION (MAXDOP 1);
GO